package solutions.bitbadger.documents

/**
 * A comparison against a field in a JSON document
 *
 * @property op The operation for the field comparison
 * @property value The value against which the comparison will be made
 */
class Comparison<T>(val op: Op, val value: T) {

    /** Is the value for this comparison a numeric value? */
    val isNumeric: Boolean
        get() =
            if (op == Op.IN || op == Op.BETWEEN) {
                val values = value as? Collection<*>
                if (values.isNullOrEmpty()) {
                    false
                } else {
                    val first = values.elementAt(0)
                    first is Byte || first is Short || first is Int || first is Long
                }
            } else {
                value is Byte || value is Short || value is Int || value is Long
            }
}
